/*
 * File functions
 *
 * Copyright (c) 2010-2012, Joachim Metz <jbmetz@users.sourceforge.net>
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBQCOW_INTERNAL_FILE_H )
#define _LIBQCOW_INTERNAL_FILE_H

#include <common.h>
#include <types.h>

#include <liberror.h>

#include "libqcow_cluster_table.h"
#include "libqcow_extern.h"
#include "libqcow_io_handle.h"
#include "libqcow_libbfio.h"
#include "libqcow_libfcache.h"
#include "libqcow_libfdata.h"

#if defined( _MSC_VER ) || defined( __BORLANDC__ ) || defined( __MINGW32_VERSION ) || defined( __MINGW64_VERSION_MAJOR )

/* This inclusion is needed otherwise some linkers
 * mess up exporting the legacy and metadata functions
 */
#include "libqcow_metadata.h"
#endif

#if defined( __cplusplus )
extern "C" {
#endif

typedef struct libqcow_internal_file libqcow_internal_file_t;

struct libqcow_internal_file
{
	/* The IO handle
	 */
	libqcow_io_handle_t *io_handle;

	/* The file IO handle
	 */
	libbfio_handle_t *file_io_handle;

	/* Value to indicate if the file IO handle was created inside the library
	 */
	uint8_t file_io_handle_created_in_library;

	/* The level 1 table
	 */
	libqcow_cluster_table_t *level1_table;

	/* The level 2 table vector
	 */
	libfdata_vector_t *level2_table_vector;

	/* The level2 table cache
	 */
	libfcache_cache_t *level2_table_cache;

	/* The cluster block vector
	 */
	libfdata_vector_t *cluster_block_vector;

	/* The cluster block cache
	 */
	libfcache_cache_t *cluster_block_cache;

	/* Value to indicate if abort was signalled
	 */
	int abort;
};

LIBQCOW_EXTERN \
int libqcow_file_initialize(
     libqcow_file_t **file,
     liberror_error_t **error );

LIBQCOW_EXTERN \
int libqcow_file_free(
     libqcow_file_t **file,
     liberror_error_t **error );

LIBQCOW_EXTERN \
int libqcow_file_signal_abort(
     libqcow_file_t *file,
     liberror_error_t **error );

LIBQCOW_EXTERN \
int libqcow_file_open(
     libqcow_file_t *file,
     const char *filename,
     int access_flags,
     liberror_error_t **error );

#if defined( HAVE_WIDE_CHARACTER_TYPE )
LIBQCOW_EXTERN \
int libqcow_file_open_wide(
     libqcow_file_t *file,
     const wchar_t *filename,
     int access_flags,
     liberror_error_t **error );
#endif

LIBQCOW_EXTERN \
int libqcow_file_open_file_io_handle(
     libqcow_file_t *file,
     libbfio_handle_t *file_io_handle,
     int access_flags,
     liberror_error_t **error );

LIBQCOW_EXTERN \
int libqcow_file_close(
     libqcow_file_t *file,
     liberror_error_t **error );

int libqcow_file_open_read(
     libqcow_internal_file_t *internal_file,
     liberror_error_t **error );

LIBQCOW_EXTERN \
ssize_t libqcow_file_read_buffer(
         libqcow_file_t *file,
         void *buffer,
         size_t buffer_size,
         liberror_error_t **error );

#ifdef TODO
LIBQCOW_EXTERN \
ssize_t libqcow_file_write_buffer(
         libqcow_file_t *file,
         const void *buffer,
         size_t buffer_size,
         liberror_error_t **error );
#endif

LIBQCOW_EXTERN \
off64_t libqcow_file_seek_offset(
         libqcow_file_t *file,
         off64_t offset,
         int whence,
         liberror_error_t **error );

LIBQCOW_EXTERN \
int libqcow_file_get_offset(
     libqcow_file_t *file,
     off64_t *offset,
     liberror_error_t **error );

LIBQCOW_EXTERN \
int libqcow_file_get_format_version(
     libqcow_file_t *file,
     uint32_t *format_version,
     liberror_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif

